@@ -1,6 +1,6 @@
 
 
-lmSLX <- function(formula, data = list(), listw, na.action, weights=NULL, Durbin=TRUE, zero.policy=NULL) {
+lmSLX <- function(formula, data = list(), listw, na.action, weights=NULL, Durbin=TRUE, zero.policy=NULL, return_impacts=TRUE) {
         if (is.null(zero.policy))
             zero.policy <- get("zeroPolicy", envir = .spatialregOptions)
         stopifnot(is.logical(zero.policy))
@@ -18,6 +18,7 @@ lmSLX <- function(formula, data = list(), listw, na.action, weights=NULL, Durbin
         if (attr(mt, "intercept") == 1 && !any(attr(mt, "factors") == 1)) {
             stop("intercept-only model, Durbin invalid")
         }
+	have_factor_preds <- have_factor_preds_mf(mf)
 	na.act <- attr(mf, "na.action")
 	if (!inherits(listw, "listw")) stop("No neighbourhood list")
 	if (listw$style == "M") warning("missing spatial weights style")
@@ -45,59 +46,24 @@ lmSLX <- function(formula, data = list(), listw, na.action, weights=NULL, Durbin
         if (is.null(weights)) weights <- rep(as.numeric(1), n)
         if (any(is.na(weights))) stop("NAs in weights")
         if (any(weights < 0)) stop("negative weights")
-        dvars <- c(NCOL(x), 0L)
-        prefix <- "lag"
-        if (isTRUE(Durbin)) {
-            WX <- create_WX(x, listw, zero.policy=zero.policy,
-               prefix=prefix)
-        } else if (is.formula(Durbin)) {
-            data1 <- data
-            if (!is.null(na.act) && (inherits(na.act, "omit") ||
-                inherits(na.act, "exclude"))) {
-                data1 <- data1[-c(na.act),]
-            }
-            dmf <- lm(Durbin, data1, na.action=na.fail, 
-	        method="model.frame")
-#	    dmf <- lm(Durbin, data, na.action=na.action, 
-#	         method="model.frame")
-            fx <- try(model.matrix(Durbin, dmf), silent=TRUE)
-            if (inherits(fx, "try-error")) 
-                 stop("Durbin variable mis-match")
-            WX <- create_WX(fx, listw, zero.policy=zero.policy,
-                prefix=prefix)
-            inds <- match(substring(colnames(WX), 5,
-	        nchar(colnames(WX))), colnames(x))
-            if (anyNA(inds)) {
-              wna <- which(is.na(inds)) #TR: continue if Durbin has intercept, but formula has not
-              if (length(wna) == 1 && grepl("Intercept", colnames(WX)[wna])
-                 && attr(terms(formula), "intercept") == 0
-                 && attr(terms(Durbin), "intercept") == 1) {
-                inds <- inds[-wna]
-              } else{
-                stop("WX variables not in X: ",
-                     paste(substring(colnames(WX), 5,
-                     nchar(colnames(WX)))[is.na(inds)], collapse=" "))
-              }
-            } 
-            icept <- grep("(Intercept)", colnames(x))
-            iicept <- length(icept) > 0L
-            if (iicept) {
-                xn <- colnames(x)[-1]
-            } else {
-                xn <- colnames(x)
-            }
-            wxn <- substring(colnames(WX), nchar(prefix)+2,
-                nchar(colnames(WX)))
-            zero_fill <- length(xn) + (which(!(xn %in% wxn)))
-        } else stop("Durbin argument neither TRUE nor formula")
-        dvars <- c(NCOL(x), NCOL(WX))
-        if (is.formula(Durbin)) {
-            attr(dvars, "f") <- Durbin
-            attr(dvars, "inds") <- inds
-            attr(dvars, "zero_fill") <- zero_fill
+#        dvars <- c(NCOL(x), 0L)
+        if (!(isTRUE(Durbin) || is.formula(Durbin))) {
+            stop("Durbin argument neither TRUE nor formula")
+        } else {
+            res <- create_Durbin(Durbin=Durbin, 
+                have_factor_preds=have_factor_preds, x=x, listw=listw,
+                zero.policy=zero.policy, data=data, na.act=na.act,
+                formula=formula)
+            x <- res$x
+            dvars <- res$dvars
+            inds <-attr(dvars, "inds") 
+            xn <- attr(dvars, "xn")
+            wxn <- attr(dvars, "wxn")
+            zero_fill <- attr(dvars, "zero_fill")
+            formula_durbin_factors <- attr(dvars, "formula_durbin_factors")
+            attr(dvars, "xn") <- NULL
+            attr(dvars, "wxn") <- NULL
         }
-	x <- cbind(x, WX)
-	rm(WX)
 #        WX <- create_WX(x, listw, zero.policy=zero.policy, prefix="lag")
 #        x <- cbind(x, WX)
 # 180128 Mark L. Burkey summary.lm error for SlX object
@@ -108,17 +74,19 @@ lmSLX <- function(formula, data = list(), listw, na.action, weights=NULL, Durbin
             lm.model <- lm(formula(paste("y ~ 0 + ", paste(colnames(x), collapse="+"))), data=as.data.frame(x), weights=weights)
         }
         sum_lm_model <- summary.lm(lm.model, correlation = FALSE)
+        if (any(sum_lm_model$aliased)) warning("aliased variables found")
         mixedImps <- NULL
-	K <- ifelse(isTRUE(grep("Intercept",
+        if (return_impacts) {
+	  K <- ifelse(isTRUE(grep("Intercept",
             names(coefficients(lm.model))[1]) == 1L), 2, 1)
-        if (isTRUE(Durbin)) {
-          m <- length(coefficients(lm.model))
-          m.1 <- m > 1
-          if (m.1 && K == 2) { #TR: without intercept and m.1 use m/2
+          if (isTRUE(Durbin)) {
+            m <- length(coefficients(lm.model))
+            m.1 <- m > 1
+            if (m.1 && K == 2) { #TR: without intercept and m.1 use m/2
               m2 <- (m-1)/2
-          } else {
+            } else {
               m2 <- m/2
-          }
+            }
             cm <- matrix(0, ncol=m, nrow=m2)
             if (K == 2) {
                 if (m.1) {
@@ -143,7 +111,7 @@ lmSLX <- function(formula, data = list(), listw, na.action, weights=NULL, Durbin
             }
             suppressWarnings(lc <- summary(multcomp::glht(lm.model, linfct=cm)))
             totImps <- cbind("Estimate"=lc$test$coefficients, "Std. Error"=lc$test$sigma)
-      } else if (is.formula(Durbin)) {
+        } else if (is.formula(Durbin)) {
 #FIXME
             LI <- ifelse(listw$style != "W" 
                          && attr(terms(Durbin), "intercept") == 1, 1, 0) #TR: lagged intercept if not W and in Durbin formula
@@ -195,10 +163,11 @@ lmSLX <- function(formula, data = list(), listw, na.action, weights=NULL, Durbin
                     }
                   }
                 rownames(totImps) <- xn
-        } else stop("undefined Durbin state")
-        mixedImps <- list(dirImps=dirImps, indirImps=indirImps,
+          } else stop("undefined Durbin state")
+          mixedImps <- list(dirImps=dirImps, indirImps=indirImps,
             totImps=totImps)
         
+        }
         attr(lm.model, "mixedImps") <- mixedImps
         attr(lm.model, "dvars") <- dvars
         if (is.formula(Durbin)) attr(lm.model, "Durbin") <- deparse(Durbin)
@@ -206,6 +175,7 @@ lmSLX <- function(formula, data = list(), listw, na.action, weights=NULL, Durbin
         tms <- as.character(lm.model$terms)
         attr(lm.model, "SLX_call") <- paste(y_name, " ", tms[1], " ", tms[3],
             ", data = ", d_name, ", listw = ", listw_name, sep="")
+        attr(lm.model, "have_factor_preds") <- have_factor_preds
         class(lm.model) <- c("SlX", class(lm.model))
         lm.model
 }
@@ -302,27 +272,69 @@ impacts.SlX <- function(obj, ...) {
     stopifnot(!is.null(attr(obj, "mixedImps")))
     n <- nrow(obj$model)
     k <- obj$qr$rank
-    impactsWX(attr(obj, "mixedImps"), n, k, type="SlX", method="glht")
+    hfp <- attr(obj, "have_factor_preds")
+    if (is.null(hfp)) hfp <- FALSE
+    impactsWX(attr(obj, "mixedImps"), n, k, type="SlX", method="glht",
+        have_factor_preds=hfp)
 }
 
-
-impactsWX <- function(obj, n, k, type="SlX", method="glht") {
+impactsWX <- function(obj, n, k, type="SlX", method="glht", have_factor_preds=FALSE) {
     imps <- lapply(obj, function(x) x[, 1])
+    bnames <- rownames(obj[[1]])
+    bnames <- update_bnames(bnames, have_factor_preds=have_factor_preds)
     names(imps) <- c("direct", "indirect", "total")
-    attr(imps, "bnames") <- rownames(obj[[1]])
+    attr(imps, "bnames") <- bnames
     ses <- lapply(obj, function(x) x[, 2])
     names(ses) <- c("direct", "indirect", "total")
-    attr(ses, "bnames") <- rownames(obj[[1]])
+    attr(ses, "bnames") <- bnames
     res <- list(impacts=imps, se=ses)
     attr(res, "n") <- n
     attr(res, "k") <- k
     attr(res, "type") <- type
     attr(res, "method") <- method
-    attr(res, "bnames") <- rownames(obj[[1]])
+    attr(res, "bnames") <- bnames
     class(res) <- "WXimpact"
     res
 }
 
+update_bnames <- function(bnames, have_factor_preds=FALSE) {
+    b_suffix <- rep("dy/dx", length(bnames))
+    stopifnot(!is.null(have_factor_preds))
+    if (have_factor_preds) {
+        factnames <- attr(have_factor_preds, "factnames")
+        for (pred in seq(along=factnames)) {
+            npred <- grep(paste0("^", factnames[pred], ".*"), bnames)
+            b_suffix[npred] <- "(F)"
+#=======
+#        xlevels <- attr(have_factor_preds, "xlevels")
+#        contrasts <- attr(have_factor_preds, "pred_contrasts")
+#        for (pred in seq(along=factnames)) {
+#            npred <- grep(factnames[pred], bnames)
+#            xlpred <- xlevels[[pred]]
+#            cpred <- contrasts[[pred]]
+#            if (length(npred) >= length(xlpred)) {
+#                b_suffix[npred] <- ""
+#            } else {
+#                if (cpred == "contr.treatment") {
+#                    b_suffix[npred] <- xlpred[-1]
+#                } else if (cpred == "code_control") {
+#                    b_suffix[npred] <- paste(xlpred[-1], "vs.", xlpred[1])
+#                } else if (cpred == "code_diff") {
+#                    b_suffix[npred] <- paste(xlpred[-1], "vs.",
+#                        xlpred[-length(xlpred)])
+#                } else if (cpred == "contr.poly") {
+#                    b_suffix[npred] <- "poly"
+#                } else {
+#                    b_suffix[npred] <- ""
+#                }
+#            }
+#>>>>>>> ddd1e233b0387bd18bd409837b00671295e6001d
+        }
+    }
+    bnames <- paste(bnames, b_suffix)
+    bnames
+}
+
 
 print.WXimpact <- function(x, ...) {
     mat <- lagImpactMat(x$impacts)
@@ -427,4 +439,67 @@ create_WX <- function(x, listw, zero.policy=NULL, prefix="") {
         WX
 }
 
+create_Durbin <- function(Durbin, have_factor_preds, x, listw, zero.policy, 
+        data, na.act, formula) {
+        prefix <- "lag"
+        if (isTRUE(Durbin)) {
+            if (have_factor_preds) warn_factor_preds(have_factor_preds)
+            WX <- create_WX(x, listw, zero.policy=zero.policy,
+               prefix=prefix)
+        } else if (is.formula(Durbin)) {
+            data1 <- data
+            if (!is.null(na.act) && (inherits(na.act, "omit") ||
+                inherits(na.act, "exclude"))) {
+                data1 <- data1[-c(na.act),]
+            }
+            dmf <- lm(Durbin, data1, na.action=na.fail, 
+	        method="model.frame")
+	    formula_durbin_factors <- have_factor_preds_mf(dmf)
+            if (formula_durbin_factors)
+                warn_factor_preds(formula_durbin_factors)
+#	    dmf <- lm(Durbin, data, na.action=na.action, 
+#	         method="model.frame")
+            fx <- try(model.matrix(Durbin, dmf), silent=TRUE)
+            if (inherits(fx, "try-error")) 
+                 stop("Durbin variable mis-match")
+            WX <- create_WX(fx, listw, zero.policy=zero.policy,
+                prefix=prefix)
+            inds <- match(substring(colnames(WX), 5,
+	        nchar(colnames(WX))), colnames(x))
+            if (anyNA(inds)) {
+              wna <- which(is.na(inds)) #TR: continue if Durbin has intercept, but formula has not
+              if (length(wna) == 1 && grepl("Intercept", colnames(WX)[wna])
+                 && attr(terms(formula), "intercept") == 0
+                 && attr(terms(Durbin), "intercept") == 1) {
+                inds <- inds[-wna]
+              } else{
+                stop("WX variables not in X: ",
+                     paste(substring(colnames(WX), 5,
+                     nchar(colnames(WX)))[is.na(inds)], collapse=" "))
+              }
+            } 
+            icept <- grep("(Intercept)", colnames(x))
+            iicept <- length(icept) > 0L
+            if (iicept) {
+                xn <- colnames(x)[-1]
+            } else {
+                xn <- colnames(x)
+            }
+            wxn <- substring(colnames(WX), nchar(prefix)+2,
+                nchar(colnames(WX)))
+            zero_fill <- length(xn) + (which(!(xn %in% wxn)))
+        } 
+        dvars <- c(NCOL(x), NCOL(WX))
+        if (is.formula(Durbin)) {
+            attr(dvars, "f") <- Durbin
+            attr(dvars, "inds") <- inds
+            attr(dvars, "xn") <- xn
+            attr(dvars, "wxn") <- wxn
+            attr(dvars, "zero_fill") <- zero_fill
+            attr(dvars, "formula_durbin_factors") <- formula_durbin_factors
+        }
+	x <- cbind(x, WX)
+	rm(WX)
+        list(x=x, dvars=dvars)
+}
 
