@@ -33,6 +33,7 @@ GMerrorsar <- function(#W, y, X,
         if (!inherits(formula, "formula")) formula <- as.formula(formula)
 	mt <- terms(formula, data = data)
 	mf <- lm(formula, data, na.action=na.action, method="model.frame")
+        have_factor_preds <- have_factor_preds_mf(mf)
 	na.act <- attr(mf, "na.action")
 	if (!is.null(na.act)) {
 	    subset <- !(1:length(listw$neighbours) %in% na.act)
@@ -234,6 +235,7 @@ GMerrorsar <- function(#W, y, X,
 	}
 	if (!is.null(na.act))
 		ret$na.action <- na.act
+        attr(ret, "have_factor_preds") <- have_factor_preds
 	ret
 }
 
@@ -411,6 +413,8 @@ impacts.Gmsar <- function(obj, ..., n=NULL, tr=NULL, R=NULL, listw=NULL,
     Sigma <- obj$secstep_var
     irho <- 1
     drop2beta <- 1
+    bnames <- update_bnames(bnames,
+        have_factor_preds=attr(obj, "have_factor_preds"))
     res <- intImpacts(rho=rho, beta=beta, P=P, n=n, mu=mu, Sigma=Sigma,
         irho=irho, drop2beta=drop2beta, bnames=bnames, interval=NULL,
         type="lag", tr=tr, R=R, listw=listw, evalues=evalues, tol=tol,
@@ -518,9 +522,9 @@ impacts.Gmsar <- function(obj, ..., n=NULL, tr=NULL, R=NULL, listw=NULL,
 ####SARAR model
 
 gstsls<-function (formula, data = list(), listw, listw2=NULL,
- na.action = na.fail, zero.policy = attr(listw, "zero.policy"), pars=NULL, scaleU=FALSE,
- control = list(), verbose = NULL, method = "nlminb", robust = FALSE,
- legacy = FALSE, W2X = TRUE ) 
+ na.action = na.fail, zero.policy = attr(listw, "zero.policy"), pars=NULL,
+ scaleU=FALSE, control = list(), verbose = NULL, method = "nlminb",
+ robust = FALSE, legacy = FALSE, W2X = TRUE, sig2n_k=FALSE) 
 {
 	
 	
@@ -543,6 +547,7 @@ gstsls<-function (formula, data = list(), listw, listw2=NULL,
     if (!inherits(formula, "formula")) formula <- as.formula(formula)
     mt <- terms(formula, data = data)
     mf <- lm(formula, data, na.action = na.fail, method = "model.frame")
+    have_factor_preds <- have_factor_preds_mf(mf)
     na.act <- attr(mf, "na.action")
     cl <- match.call()
     if (!is.null(na.act)) {
@@ -590,7 +595,7 @@ gstsls<-function (formula, data = list(), listw, listw2=NULL,
         }
 
         instr <- cbind(WX, WWX)
-        firststep <- tsls(y = y, yend = wy, X = x, Zinst = instr, robust = robust, legacy = legacy)
+        firststep <- tsls(y = y, yend = wy, X = x, Zinst = instr, robust = robust, legacy = legacy, sig2n_k=sig2n_k)
 
         ukp <- residuals(firststep)
 
@@ -641,7 +646,7 @@ gstsls<-function (formula, data = list(), listw, listw2=NULL,
         colnames(xt) <- xcolnames
         colnames(wyt) <- c("Rho_Wy")
         secstep <- tsls(y = yt, yend = wyt, X = xt, Zinst = instr,
-            robust = robust, legacy = legacy)
+            robust = robust, legacy = legacy, sig2n_k=sig2n_k)
 	rho<-secstep$coefficients[1]
 	coef.sac<-secstep$coefficients
 	rest.se <- sqrt(diag(secstep$var))
@@ -670,6 +675,7 @@ gstsls<-function (formula, data = list(), listw, listw2=NULL,
     }
         
     if (!is.null(na.act)) ret$na.action <- na.act
+    attr(ret, "have_factor_preds") <- have_factor_preds
     ret
 }
 
